/*******************************************************************************
  File: FactoryCollection.hxx
  Copyright (c) 1999 Bleading Edge Software Technologies, Inc.

  This Software is distributed under the open source, free software model.
  Permission to use, modify, and distribute this Software, including
  incorporating it into proprietary software, is granted without fee, 
  provided this copyright notice appear in all copies of the code.
  You are under no obligation to distribute source code which is derived from
  or uses this Software. You may not do anything however, to prevent the 
  continued distribution of this Software under an open source distribution
  model. This includes, but is not limited to, such things as copyrighting
  or claiming authorship of the Software.
  This Software is distributed in the hope that it will be useful, but it is
  provide "as is", WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

//$-----------------------------------------------------------------------------
$Header: $
  Author: Jack W. Reeves

//!----------------------------------------------------------------------------|
Description:
  Implements the FactoryCollection template
  
//#----------------------------------------------------------------------------|
Notes:

//%****************************************************************************/

// My header
#include "FactoryCollection.h"

// Utilities
#include "check.h"


namespace Util {

// static data
template<typename BaseType>
FactoryCollection<BaseType>::TypeNameMap	FactoryCollection<BaseType>::_typeNames;

template<typename BaseType>
FactoryCollection<BaseType>::FactoryMap		FactoryCollection<BaseType>::_factories;


// lookup type names
template<typename BaseType>
std::string
FactoryCollection<BaseType>::find(const std::type_info& type)
{
	TypeNameMap::const_iterator it = _typeNames.find(&type);
	if (it == _typeNames.end()) 
		raise_invalid_argument("type_info not registered", "");
	return (*it).second;
}

template<typename BaseType>
std::string
FactoryCollection<BaseType>::find(const std::type_info& type, std::nothrow_t)
{
	TypeNameMap::const_iterator it = _typeNames.find(&type);
	return (it == _typeNames.end()) ? std::string() : (*it).second;
}


// lookup factory
template<typename BaseType>
FactoryCollection<BaseType>::Factory&
FactoryCollection<BaseType>::find(const std::string& name)
{
	FactoryMap::const_iterator it = _factories.find(name);
	if (it == _factories.end())
		raise_invalid_argument("type name not registered", "");
	return *(*it).second;
}


template<typename BaseType>
FactoryCollection<BaseType>::Factory*
FactoryCollection<BaseType>::find(const std::string& name, std::nothrow_t)
{
	FactoryMap::const_iterator it = _factories.find(name);
	return (it == _factories.end()) ? null : (*it).second;
}


// register type
template<typename BaseType>
template<typename ConcreteClass>
void
FactoryCollection<BaseType>::register_type(const std::string& name)
{
	std::pair<TypeNameMap::iterator, bool> rtn1 = _typeNames.insert( TypeNameMap::value_type(&typeid(ConcreteClass), name) );
	if (not rtn1.second)
		raise_domain_error("type info already registered, can not be registered again", "");
	std::pair<FactoryMap::iterator, bool> rtn2 = _factories.insert( FactoryMap::value_type(name, null) );
	if (not rtn2.second) {
		_typeNames.erase(rtn1.first);
		raise_domain_error("type name already registered, can not be registered again", "");
	}
	(*rtn2.first).second = new TFactory<ConcreteClass>;
}


template<typename BaseType>
template<typename ConcreteClass>
bool
FactoryCollection<BaseType>::register_type(const std::string& name, std::nothrow_t)
{
	std::pair<TypeNameMap::iterator, bool> rtn1 = _typeNames.insert( TypeNameMap::value_type(&typeid(ConcreteClass), name) );
	if (not rtn1.second) return false;
	std::pair<FactoryMap::iterator, bool> rtn2 = _factories.insert( FactoryMap::value_type(name, null) );
	if (not rtn2.second) {
		_typeNames.erase(rtn1.first);
		return false;
	}
	(*rtn2.first).second = new TFactory<ConcreteClass>;
}


// TFactory implementation
template<typename BaseType>
template<typename ConcreteClass>
BaseType*
FactoryCollection<BaseType>::TFactory<ConcreteClass>::make() const
{
	return new ConcreteClass;
}


} //> namespace Util
