#ifndef PERL_EXCEPTIONS_H
#define PERL_EXCEPTIONS_H

#include <string>
#include <exception>
#include <typeinfo>
 
// A class that is thrown for a Perl evaluation error
class PerlEvalError: public exception {
private:
    string err_str;

public:
    PerlEvalError(const string& _err_str) throw() : err_str(_err_str) {}
    virtual ~PerlEvalError() throw() {}
    virtual const char* what() const throw() { return err_str.c_str(); }

private:
    PerlEvalError();       // not allowed
};

// A class that is thrown when a PerlValue is cast to a type that it is not
class PerlBadCast: public bad_cast {
private:
    string err_str;

public:
    PerlBadCast(const string& _err_str) throw() : err_str(_err_str) {}
    virtual ~PerlBadCast() throw() {}
    virtual const char* what() const throw() { return err_str.c_str(); }

private:
    PerlBadCast();         // not allowed
};

// A class that is thrown when an index to a Perl array is out of bounds
class PerlOutOfBounds: public exception {
private:
    string err_str;

public:
    PerlOutOfBounds(const string& _err_str) throw() : err_str(_err_str) {}
    virtual ~PerlOutOfBounds() throw() {}
    virtual const char* what() const throw() { return err_str.c_str(); }

private:
    PerlOutOfBounds();     // not allowed
};

#endif
